@@ -58,6 +58,8 @@ Plugins required:
       (default '-1, all')
     * **days-to-keep** (`int`): For how many days should a build be kept.
       (default '-1, forever')
+    * **abort-builds** (`bool`): Abort all pending or ongoing builds for removed
+      SCM heads (i.e. deleted branches). (default false)
     * **script-path** (`str`): Path to Jenkinsfile, relative to workspace.
       (default 'Jenkinsfile')
     * **script-id** (`str`): Script id from the global Jenkins script store
@@ -84,8 +86,8 @@ import jenkins_jobs.modules.helpers as helpers
 import six
 
 from jenkins_jobs.modules.scm import git_extensions
-from jenkins_jobs.errors import InvalidAttributeError
-from jenkins_jobs.errors import JenkinsJobsException
+from jenkins_jobs.errors import InvalidAttributeError, MissingAttributeError
+from jenkins_jobs.errors import Context, JenkinsJobsException
 from jenkins_jobs.xml_config import remove_ignorable_whitespace
 
 logger = logging.getLogger(str(__name__))
@@ -220,6 +222,7 @@ class WorkflowMultiBranch(jenkins_jobs.modules.base.Base):
             ("prune-dead-branches", "pruneDeadBranches", True, [True, False]),
             ("days-to-keep", "daysToKeep", -1),
             ("number-to-keep", "numToKeep", -1),
+            ("abort-builds", "abortBuilds", False, [True, False]),
         ]
         helpers.convert_mapping_to_xml(ois, data, ois_mapping)
 
@@ -427,8 +430,14 @@ def bitbucket_scm(xml_parent, data):
     :extensions:
 
         * **clean** (`dict`)
-            * **after** (`bool`) - Clean the workspace after checkout
-            * **before** (`bool`) - Clean the workspace before checkout
+            * **after** (`dict`) - Clean the workspace after checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                  submodules and any other subdirectories which contain .git directories
+                  (default false)
+            * **before** (`dict`) - Clean the workspace before checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                  submodules and any other subdirectories which contain .git directories
+                  (default false)
         * **prune** (`bool`) - Prune remote branches (default false)
         * **shallow-clone** (`bool`) - Perform shallow clone (default false)
         * **sparse-checkout** (dict)
@@ -653,6 +662,9 @@ def gerrit_scm(xml_parent, data):
         (default '*')
     :arg str excludes: Comma-separated list of branches to be excluded.
         (default '')
+    :arg str head-filter-regex: A regular expression for filtering
+        discovered source branches. Requires the :jenkins-plugins:`SCM API
+        Plugin <scm-api>`.
     :arg list build-strategies: Provides control over whether to build a branch
         (or branch like things such as change requests and tags) whenever it is
         discovered initially or a change from the previous revision has been
@@ -678,8 +690,14 @@ def gerrit_scm(xml_parent, data):
     :extensions:
 
         * **clean** (`dict`)
-            * **after** (`bool`) - Clean the workspace after checkout
-            * **before** (`bool`) - Clean the workspace before checkout
+            * **after** (`dict`) - Clean the workspace after checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                  submodules and any other subdirectories which contain .git directories
+                  (default false)
+            * **before** (`dict`) - Clean the workspace before checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                  submodules and any other subdirectories which contain .git directories
+                  (default false)
         * **prune** (`bool`) - Prune remote branches (default false)
         * **shallow-clone** (`bool`) - Perform shallow clone (default false)
         * **sparse-checkout** (dict)
@@ -786,6 +804,10 @@ def gerrit_scm(xml_parent, data):
     if data.get("build-strategies", None):
         build_strategies(xml_parent, data)
 
+    if data.get("head-filter-regex", None):
+        rshf = XML.SubElement(traits, "jenkins.scm.impl.trait.RegexSCMHeadFilterTrait")
+        XML.SubElement(rshf, "regex").text = data.get("head-filter-regex")
+
     # FilterChecks Trait
     checks = data.get("filter-checks", None)
     if checks:
@@ -848,8 +870,14 @@ def git_scm(xml_parent, data):
     :extensions:
 
         * **clean** (`dict`)
-            * **after** (`bool`) - Clean the workspace after checkout
-            * **before** (`bool`) - Clean the workspace before checkout
+            * **after** (`dict`) - Clean the workspace after checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                  submodules and any other subdirectories which contain .git directories
+                  (default false)
+            * **before** (`dict`) - Clean the workspace before checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                  submodules and any other subdirectories which contain .git directories
+                  (default false)
         * **prune** (`bool`) - Prune remote branches (default false)
         * **shallow-clone** (`bool`) - Perform shallow clone (default false)
         * **sparse-checkout** (dict)
@@ -983,10 +1011,12 @@ def github_scm(xml_parent, data):
         discovered initially or a change from the previous revision has been
         detected. (optional)
         Refer to :func:`~build_strategies <build_strategies>`.
-    :arg str notification-context: Change the default GitHub check notification
-        context from "continuous-integration/jenkins/SUFFIX" to a custom text,
+    :arg dict notification-context: Change the default GitHub check notification
+        context from "continuous-integration/jenkins/SUFFIX" to a custom label / suffix.
+        (set a label and suffix to true or false, optional)
         Requires the :jenkins-plugins:`Github Custom Notification Context SCM
         Behaviour <github-scm-trait-notification-context>`.
+        Refer to :func:`~add_notification_context_trait <add_notification_context_trait>`.
     :arg dict property-strategies: Provides control over how to build a branch
         (like to disable SCM triggering or to override the pipeline durability)
         (optional)
@@ -1004,8 +1034,14 @@ def github_scm(xml_parent, data):
     :extensions:
 
         * **clean** (`dict`)
-            * **after** (`bool`) - Clean the workspace after checkout
-            * **before** (`bool`) - Clean the workspace before checkout
+            * **after** (`dict`) - Clean the workspace after checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                    submodules and any other subdirectories which contain .git directories
+                    (default false)
+            * **before** (`dict`) - Clean the workspace before checkout
+                * **remove-stale-nested-repos** (`bool`) - Deletes untracked
+                    submodules and any other subdirectories which contain .git directories
+                    (default false)
         * **prune** (`bool`) - Prune remote branches (default false)
         * **shallow-clone** (`bool`) - Perform shallow clone (default false)
         * **sparse-checkout** (dict)
@@ -1158,15 +1194,8 @@ def github_scm(xml_parent, data):
     if data.get("build-strategies", None):
         build_strategies(xml_parent, data)
 
-    if data.get("notification-context", None):
-        rshf = XML.SubElement(
-            traits,
-            "org.jenkinsci.plugins.githubScmTraitNotificationContext."
-            "NotificationContextTrait",
-        )
-        XML.SubElement(rshf, "contextLabel").text = data.get("notification-context")
-        XML.SubElement(rshf, "typeSuffix").text = "true"
-
+    add_github_checks_traits(traits, data)
+    add_notification_context_trait(traits, data)
     add_filter_by_name_wildcard_behaviors(traits, data)
 
     # handle the default git extensions like:
@@ -1431,33 +1460,58 @@ def property_strategies(xml_parent, data):
         * **all-branches** (list): A list of property strategy definitions
             for use with all branches.
 
-            * **suppress-scm-triggering** (bool): Suppresses automatic SCM
-                triggering (optional)
+            * **suppress-scm-triggering** (dict):
+                Suppresses automatic SCM triggering (optional).
+
+                * **branch-regex** (str):
+                    Regex matching branch names.
+                    Only these branches will be affected by the selected
+                    suppression strategy. Default ``^$``.
+                * **suppress-strategy** (str):
+                    Select what to suppress for branches matching the regex.
+                    Valid values: ``suppress-nothing`` (default),
+                    ``suppress-branch-indexing``, or ``suppress-webhooks``.
+
+                    .. note::
+                        Using ``suppress-scm-triggering: true`` is deprecated
+                        since Branch API 2.1045.v4ec3ed07b_e4f.
             * **pipeline-branch-durability-override** (str): Set a custom
                 branch speed/durability level. Valid values:
                 performance-optimized, survivable-nonatomic, or
                 max-survivability (optional)
                 Requires the :jenkins-plugins:`Pipeline Multibranch Plugin
                 <workflow-multibranch>`
-            * **trigger-build-on-pr-comment** (str): The comment body to
+            * **trigger-build-on-pr-comment** (str or dict): The comment body to
                 trigger a new build for a PR job when it is received. This
-                is compiled as a case insensitive regular expression, so
+                is compiled as a case-insensitive regular expression, so
                 use ``".*"`` to trigger a build on any comment whatsoever.
                 (optional)
+                If dictionary syntax is used, the option requires 2 fields:
+                ``comment`` with the comment body and ``allow-untrusted-users``
+                (bool) causing the plugin to skip checking if the comment author
+                is a collaborator of the GitHub project.
                 Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
                 <github-pr-comment-build>`
-            * **trigger-build-on-pr-review** (bool): This property will
+            * **trigger-build-on-pr-review** (bool or dict): This property will
                 cause a job for a pull request ``(PR-*)`` to be triggered
                 immediately when a review is made on the PR in GitHub.
                 This has no effect on jobs that are not for pull requests.
                 (optional)
+                If dictionary syntax is used, the option requires
+                ``allow-untrusted-users`` (bool) causing the plugin to skip
+                checking if the review author is a collaborator of the GitHub
+                project.
                 Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
                 <github-pr-comment-build>`
-            * **trigger-build-on-pr-update** (bool): This property will
+            * **trigger-build-on-pr-update** (bool or dict): This property will
                 cause a job for a pull request ``(PR-*)`` to be triggered
                 immediately when the PR title or description is edited in
                 GitHub. This has no effect on jobs that are not for pull
                 requests. (optional)
+                If dictionary syntax is used, the option requires
+                ``allow-untrusted-users`` (bool) causing the plugin to skip
+                checking if the update author is a collaborator of the GitHub
+                project.
                 Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
                 <github-pr-comment-build>`
         * **named-branches** (dict): Named branches get different properties.
@@ -1468,33 +1522,59 @@ def property_strategies(xml_parent, data):
                 to be applied by default to all branches, unless overridden
                 by an entry in `exceptions`
 
-                * **suppress-scm-triggering** (bool): Suppresses automatic SCM
-                    triggering (optional)
+                * **suppress-scm-triggering** (dict):
+                    Suppresses automatic SCM triggering (optional).
+
+                    * **branch-regex** (str):
+                        Regex matching branch names.
+                        Only these branches will be affected by the selected
+                        suppression strategy. Default ``^$``.
+                    * **suppress-strategy** (str):
+                        Select what to suppress for branches matching the regex.
+                        Valid values: ``suppress-nothing`` (default),
+                        ``suppress-branch-indexing``, or ``suppress-webhooks``.
+
+                        .. note::
+                            Using ``suppress-scm-triggering: true`` is deprecated
+                            since Branch API 2.1045.v4ec3ed07b_e4f.
                 * **pipeline-branch-durability-override** (str): Set a custom
                     branch speed/durability level. Valid values:
                     performance-optimized, survivable-nonatomic, or
                     max-survivability (optional)
                     Requires the :jenkins-plugins:`Pipeline Multibranch Plugin
                     <workflow-multibranch>`
-                * **trigger-build-on-pr-comment** (str): The comment body to
+                * **trigger-build-on-pr-comment** (str or dict): The comment body to
                     trigger a new build for a PR job when it is received. This
-                    is compiled as a case insensitive regular expression, so
+                    is compiled as a case-insensitive regular expression, so
                     use ``".*"`` to trigger a build on any comment whatsoever.
                     (optional)
+                    If dictionary syntax is used, the option accepts 2 fields:
+                    ``comment`` (str, required) with the comment body and
+                    ``allow-untrusted-users`` (bool, optional) causing the plugin
+                    to skip checking if the comment author is a collaborator of
+                    the GitHub project.
                     Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
                     <github-pr-comment-build>`
-                * **trigger-build-on-pr-review** (bool): This property will
+                * **trigger-build-on-pr-review** (bool or dict): This property will
                     cause a job for a pull request ``(PR-*)`` to be triggered
                     immediately when a review is made on the PR in GitHub.
                     This has no effect on jobs that are not for pull requests.
                     (optional)
+                    If dictionary syntax is used, the option requires
+                    ``allow-untrusted-users`` (bool) causing the plugin to skip
+                    checking if the review author is a collaborator of the GitHub
+                    project.
                     Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
                     <github-pr-comment-build>`
-                * **trigger-build-on-pr-update** (bool): This property will
+                * **trigger-build-on-pr-update** (bool or dict): This property will
                     cause a job for a pull request ``(PR-*)`` to be triggered
                     immediately when the PR title or description is edited in
                     GitHub. This has no effect on jobs that are not for pull
                     requests. (optional)
+                    If dictionary syntax is used, the option requires
+                    ``allow-untrusted-users`` (bool) causing the plugin to skip
+                    checking if the update author is a collaborator of the GitHub
+                    project.
                     Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
                     <github-pr-comment-build>`
 
@@ -1508,8 +1588,22 @@ def property_strategies(xml_parent, data):
                     * **properties** (list): A list of properties to apply to
                         this branch.
 
-                        * **suppress-scm-triggering** (bool): Suppresses
-                            automatic SCM triggering (optional)
+                        * **suppress-scm-triggering** (dict):
+                            Suppresses automatic SCM triggering (optional).
+
+                            * **branch-regex** (str):
+                                Regex matching branch names.
+                                Only these branches will be affected by the selected
+                                suppression strategy. Default ``^$``.
+                            * **suppress-strategy** (str):
+                                Select what to suppress for branches matching the regex.
+                                Valid values: ``suppress-nothing`` (default),
+                                ``suppress-branch-indexing``, or ``suppress-webhooks``.
+
+                                .. note::
+                                    Using ``suppress-scm-triggering: true`` is deprecated
+                                    since Branch API 2.1045.v4ec3ed07b_e4f.
+
                         * **pipeline-branch-durability-override** (str): Set a
                             custom branch speed/durability level. Valid values:
                             performance-optimized, survivable-nonatomic, or
@@ -1679,14 +1773,49 @@ def apply_property_strategies(props_elem, props_list):
         ]
     )
 
+    # valid options for suppress SCM trigger property
+    # strategy name says what's blocked
+    sst_map = collections.OrderedDict(
+        [
+            ("suppress-nothing", "NONE"),
+            ("suppress-webhooks", "EVENTS"),
+            ("suppress-branch-indexing", "INDEXING"),
+        ]
+    )
+
     for dbs_list in props_list:
 
-        if dbs_list.get("suppress-scm-triggering", False):
-            XML.SubElement(
+        sst_val = dbs_list.get("suppress-scm-triggering", False)
+        if sst_val:
+            sst_elem = XML.SubElement(
                 props_elem,
                 "".join([basic_property_strategies, ".NoTriggerBranchProperty"]),
             )
 
+            if isinstance(sst_val, dict):
+                if "branch-regex" not in sst_val:
+                    raise MissingAttributeError("suppress-scm-triggering[branch-regex]")
+                if "suppression-strategy" not in sst_val:
+                    raise MissingAttributeError(
+                        "suppress-scm-triggering[suppression-strategy]"
+                    )
+
+                sst_ss_val = sst_val["suppression-strategy"]
+                if not sst_map.get(sst_ss_val):
+                    raise InvalidAttributeError(
+                        "suppression-strategy", sst_ss_val, sst_map.keys()
+                    )
+                XML.SubElement(sst_elem, "triggeredBranchesRegex").text = sst_val[
+                    "branch-regex"
+                ]
+                XML.SubElement(sst_elem, "strategy").text = sst_map[sst_ss_val]
+
+            elif isinstance(sst_val, bool):
+                # no sub-elements in this case
+                pass
+            else:
+                raise InvalidAttributeError("suppress-scm-triggering", sst_val)
+
         pbdo_val = dbs_list.get("pipeline-branch-durability-override", None)
         if pbdo_val:
             if not pbdo_map.get(pbdo_val):
@@ -1707,14 +1836,45 @@ def apply_property_strategies(props_elem, props_list):
                 "".join([pr_comment_build, ".TriggerPRCommentBranchProperty"]),
                 {"plugin": "github-pr-comment-build"},
             )
-            XML.SubElement(tbopc_elem, "commentBody").text = tbopc_val
+            if isinstance(tbopc_val, dict):
+                if "comment" not in tbopc_val:
+                    raise MissingAttributeError(
+                        "trigger-build-on-pr-comment[comment]",
+                        pos=dbs_list.key_pos.get("trigger-build-on-pr-comment"),
+                    )
+                XML.SubElement(tbopc_elem, "commentBody").text = tbopc_val["comment"]
+                if tbopc_val.get("allow-untrusted-users", False):
+                    XML.SubElement(tbopc_elem, "allowUntrusted").text = "true"
+            elif isinstance(tbopc_val, str):
+                XML.SubElement(tbopc_elem, "commentBody").text = tbopc_val
+            else:
+                attr = "trigger-build-on-pr-comment"
+                ctx = [Context(f"For attribute {attr!r}", dbs_list.key_pos.get(attr))]
+                raise InvalidAttributeError(
+                    attr,
+                    tbopc_val,
+                    pos=dbs_list.value_pos.get(attr),
+                    ctx=ctx,
+                )
         for opt in pcb_bool_opts:
-            if dbs_list.get(opt, False):
-                XML.SubElement(
+            opt_value = dbs_list.get(opt, None)
+            if opt_value:
+                opt_elem = XML.SubElement(
                     props_elem,
                     "".join([pr_comment_build, pcb_bool_opts.get(opt)]),
                     {"plugin": "github-pr-comment-build"},
                 )
+                if isinstance(opt_value, dict):
+                    if opt_value.get("allow-untrusted-users", False):
+                        XML.SubElement(opt_elem, "allowUntrusted").text = "true"
+                elif isinstance(opt_value, bool):
+                    # no sub-elements in this case
+                    pass
+                else:
+                    ctx = Context(f"For attribute {opt!r}", dbs_list.key_pos.get(opt))
+                    raise InvalidAttributeError(
+                        opt, opt_value, pos=dbs_list.value_pos.get(opt), ctx=[ctx]
+                    )
 
 
 def add_filter_branch_pr_behaviors(traits, data):
@@ -1856,3 +2016,100 @@ def add_filter_by_name_wildcard_behaviors(traits, data):
             wscmf_name_mapping,
             fail_required=True,
         )
+
+
+def add_notification_context_trait(traits, data):
+    """Change the default GitHub check notification context from
+    "continuous-integration/jenkins/SUFFIX" to a custom label / suffix.
+    (set a label and suffix to true or false, optional)
+
+    Requires the :jenkins-plugins:`Github Custom Notification Context SCM
+    Behaviour <github-scm-trait-notification-context>`.
+
+    :arg dict notification-context: Definition of notification-context. A
+        `label` must be specified. `suffix` may be specified with true /
+        false, default being true.
+
+        * **label** (str): The text of the context label for Github status
+            notifications.
+        * **suffix** (bool): Appends the relevant suffix to the context label
+            based on the build type. '/pr-merge', '/pr-head' or '/branch'
+            (optional, default true)
+    """
+    if data.get("notification-context", None):
+        nc_trait = XML.SubElement(
+            traits,
+            "org.jenkinsci.plugins.githubScmTraitNotificationContext."
+            "NotificationContextTrait",
+        )
+        nc = data.get("notification-context")
+        nc_trait_label = XML.SubElement(nc_trait, "contextLabel")
+        nc_trait_suffix = XML.SubElement(nc_trait, "typeSuffix")
+        if isinstance(nc, str):
+            nc_trait_label.text = nc
+            nc_trait_suffix.text = "true"
+        else:
+            nc_trait_label.text = nc.get("label")
+            nc_suffix = nc.get("suffix", None)
+            if nc_suffix is None:
+                nc_trait_suffix.text = "true"
+            elif type(nc_suffix) == bool:
+                nc_trait_suffix.text = str(nc_suffix).lower()
+            else:
+                nc_trait_suffix.text = nc_suffix
+
+
+def add_github_checks_traits(traits, data):
+    """Enable and configure the usage of GitHub Checks API
+
+    Requires the :jenkins-plugins:`Github Checks <github-checks>` plugin.
+
+    :arg dict status-checks:
+
+        * **name** (str): The text of the context label for GitHub Checks entry
+        * **skip** (bool): Skips publishing Checks (optional, default false)
+        * **skip-branch-source-notifications** (bool): Disables the default
+            option of publishing statuses through Status API
+            (optional, default false)
+        * **publish-unstable-as-neutral** (bool): Publishes UNSTABLE builds
+            as neutral (not failed) checks
+            (optional, default false)
+        * **suppress-log-output** (bool): Suppresses sending build logs to GitHub
+            (optional, default false)
+        * **suppress-progress-updates** (bool): Suppresses updating build progress
+            (optional, default false)
+        * **verbose-logs** (bool): Enables sending build console logs to GitHub
+            (optional, default false)
+    """
+    if data.get("status-checks", None):
+        status_checks_section = data["status-checks"]
+        if status_checks_section.get("verbose-logs", None):
+            sct = XML.SubElement(
+                traits,
+                "io.jenkins.plugins.checks.github.config.GitHubSCMSourceChecksTrait",
+                {"plugin": "github-checks"},
+            )
+            sct_mapping = [("verbose-logs", "verboseConsoleLog", "false")]
+            helpers.convert_mapping_to_xml(
+                sct, status_checks_section, sct_mapping, fail_required=True
+            )
+
+        status_check_trait = XML.SubElement(
+            traits,
+            "io.jenkins.plugins.checks.github.status.GitHubSCMSourceStatusChecksTrait",
+            {"plugin": "github-checks"},
+        )
+        status_check_trait_mapping = [
+            ("skip", "skip", "false"),
+            ("skip-branch-source-notifications", "skipNotifications", "false"),
+            ("publish-unstable-as-neutral", "unstableBuildNeutral", "false"),
+            ("name", "name", "Jenkins"),
+            ("suppress-log-output", "suppressLogs", "false"),
+            ("suppress-progress-updates", "skipProgressUpdates", "false"),
+        ]
+        helpers.convert_mapping_to_xml(
+            status_check_trait,
+            status_checks_section,
+            status_check_trait_mapping,
+            fail_required=True,
+        )
